//
//  PLVFloatingWebViewController.m
//  PLVWebViewDemo-iOS
//
//  Created by MissYasiky on 2022/5/13.
//  Copyright © 2022 easefun. All rights reserved.
//

#import "PLVFloatingWebViewController.h"
#import <AVKit/AVKit.h>

static NSString * const kUserDefaultEnableBackgroundPIP = @"UserDefaultEnableBackgroundPIP";

@interface PLVFloatingWebViewController ()<
PLVWebViewManagerDelegate
>

@property (nonatomic, copy) NSString *urlString;
@property (nonatomic, strong) UIView *webviewParentView; // 负责展示webview的容器视图
@property (nonatomic, strong) PLVWKWebView *mainWebView;

@end

@implementation PLVFloatingWebViewController

#pragma mark - [ Override ]

- (void)viewDidLoad {
    [super viewDidLoad];

    // 配置音频模式
    [[AVAudioSession sharedInstance] setCategory:AVAudioSessionCategoryPlayback withOptions:AVAudioSessionCategoryOptionMixWithOthers error:nil];
    [[AVAudioSession sharedInstance] setActive:YES error: nil];
    
    self.title = @"可小窗 WKWebView";
    self.view.backgroundColor = [UIColor lightTextColor];
    self.navigationController.navigationBar.translucent = NO;
    self.navigationController.navigationBar.barTintColor = [UIColor whiteColor];
    if (@available(iOS 15.0, *)) {
        UINavigationBarAppearance *appperance = [[UINavigationBarAppearance alloc] init];
        //添加背景色
        appperance.backgroundColor = [UIColor whiteColor];
        //设置字体颜色大小
        [appperance setTitleTextAttributes:@{NSForegroundColorAttributeName:[UIColor blackColor]}];
        self.navigationController.navigationBar.standardAppearance = appperance;
        self.navigationController.navigationBar.scrollEdgeAppearance = appperance;
    }

    self.webviewParentView = [[UIView alloc] init];
    self.webviewParentView.frame = self.view.bounds;
    [self.view addSubview:self.webviewParentView];
    
    /// 小窗各项配置

    // 应用内小窗样式修改
    UIView *appWindowView = [PLVWebViewManager sharedManager].appWindowView;
    appWindowView.layer.cornerRadius = 8.0f;
    appWindowView.layer.masksToBounds = YES;
    appWindowView.layer.borderColor = [PLVWVFdUtil colorFromHexString:@"#FF8D00" alpha:1].CGColor;
    appWindowView.layer.borderWidth = 2.0f;

    BOOL enable = [self readEnableBackgroundPictureInPicture];
    [PLVWebViewManager sharedManager].autoSmallWindowPlay = enable;
    
    __weak typeof(self) weakSelf = self;
    [PLVWebViewManager sharedManager].webViewConfigBlock = ^(PLVWKWebView * _Nonnull mainWebView, PLVWKWebView * _Nonnull subWebView) {
        // 添加自定义事件
        [mainWebView registerHandler:@"testObjcCallback" handler:^(id  _Nonnull data, PLVWVJBResponseCallback  _Nonnull responseCallback) {
            
        }];
        [mainWebView callHandler:@"testJavascriptHandler" data:nil responseCallback:^(id  _Nonnull responseData) {
            
        }];
        weakSelf.mainWebView = mainWebView;
    };
    
    [[PLVWebViewManager sharedManager] attachWebViewInParentView:self.webviewParentView webViewUrl:self.urlString];

    [[PLVWebViewManager sharedManager] setWindowStateChangeBlock:^(PLVWebViewFloatingWindowState windowState) {
        switch (windowState) {
            case PLVWebViewFloatingWindowState_Active:
                NSLog(@"小窗开启");
                break;
            case PLVWebViewFloatingWindowState_Restored:
                NSLog(@"小窗恢复");
                break;
            case PLVWebViewFloatingWindowState_Close:
                NSLog(@"小窗关闭");
                break;
            default:
                break;
        }
        
    }];
    
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    [PLVWebViewManager sharedManager].supportedInterfaceOrientations = UIInterfaceOrientationMaskAllButUpsideDown;
    if ([PLVWebViewManager sharedManager].floatingWindowActive) {
        [[PLVWebViewManager sharedManager] postRevertWindowToNormal];
    }
}

- (void)viewWillLayoutSubviews {
    self.webviewParentView.frame = self.view.bounds;
}

- (void)dealloc {
    [[PLVWebViewManager sharedManager] webViewDealloc];
    NSLog(@"%s", __FUNCTION__);
}

#pragma mark - [ Public Method ]

+ (instancetype)webViewControllerWithURLString:(NSString *)urlString {
    return [[PLVFloatingWebViewController alloc] initWithURLString:urlString];
}

- (instancetype)initWithURLString:(NSString *)urlString {
    self = [super init];
    if (self) {
        if(urlString && [urlString isKindOfClass:[NSString class]] && urlString.length > 0) {
            self.urlString = urlString;
        }
        
        [[PLVWebViewManager sharedManager] setupWebViewController:self];
        [PLVWebViewManager sharedManager].delegate = self;
    }
    return self;
}

#pragma mark - [ Private Method ]
- (void)saveEnableBackgroundPictureInPicture:(BOOL)enable {
    [[NSUserDefaults standardUserDefaults] setBool:enable forKey:kUserDefaultEnableBackgroundPIP];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (BOOL)readEnableBackgroundPictureInPicture {
    id enable = [[NSUserDefaults standardUserDefaults] objectForKey:kUserDefaultEnableBackgroundPIP];
    return (enable != nil) ? [enable boolValue] : YES;
}

#pragma mark - [ Delegate ]

#pragma mark PLVWebViewManagerDelegate
- (void)webviewManager:(PLVWebViewManager *)manager willJumpLink:(NSString *)link data:(NSDictionary *)data newPage:(BOOL)newPage {
    if (!newPage) {
        /// 返回上一页
        [self.navigationController popViewControllerAnimated:YES];
    }
}

- (void)webviewManagerDidRecoverWindow:(PLVWebViewManager *)manager {
    
}

- (void)webviewManagerDidCloseWindow:(PLVWebViewManager *)manager {
    
}

- (void)webviewManager:(PLVWebViewManager *)manager callAppEvent:(NSDictionary *)event callback:(void (^)(id _Nonnull))callback {
    if (!event || ![event isKindOfClass:[NSDictionary class]]) {
        return;
    }
    
    NSString *eventName = event[@"event"];
    if ([eventName isEqualToString:@"hideNavigationBar"]) {
        NSDictionary *dict = event[@"value"];
        BOOL hidden = [dict[@"hidden"] boolValue];
        [self.navigationController setNavigationBarHidden:hidden animated:YES];
        //获取状态栏的高度
        CGFloat statusHeight = [UIApplication sharedApplication].statusBarFrame.size.height;
        //获取导航栏的高度
        CGFloat navHeight = self.navigationController.navigationBar.frame.size.height;
        NSDictionary *parames = @{@"event": @"navigationBarHeight",
                                @"value": @{@"statusHeight": @(statusHeight),
                                            @"navHeight" : @(navHeight)}};
        [[PLVWebViewManager sharedManager] callHandlerCallWebViewEvent:parames];
    } else if ([eventName isEqualToString:@"share"]) {
        // 分享
        
    } else if ([eventName isEqualToString:@"collect"]) {
        // 收藏
        
    } else if ([eventName isEqualToString:@"rotateOrientation"]) {
        // 屏幕旋转
        NSDictionary *dict = event[@"value"];
        NSString *orientation = dict[@"orientation"];
        if (!(orientation && orientation.length > 0)) {
            return;
        }
        
        UIDeviceOrientation deviceOrientation = UIDeviceOrientationPortrait;
        if ([orientation isEqualToString:@"LandscapeLeft"]) {
            deviceOrientation = UIDeviceOrientationLandscapeLeft;
        } else if ([orientation isEqualToString:@"LandscapeRight"]) {
            deviceOrientation = UIDeviceOrientationLandscapeRight;
        }
        [PLVWVFdUtil changeDeviceOrientation:deviceOrientation];
    } else if ([eventName isEqualToString:@"goBack"]) {
        // 返回上一页
        [self.navigationController popViewControllerAnimated:YES];
    } else if ([eventName isEqualToString:@"getEnableBackgroundPictureInPicture"]) {
        // 获取是否开启后台画中画
        BOOL enable = [self readEnableBackgroundPictureInPicture];
        NSDictionary *parames = @{@"event": @"setEnableBackgroundPictureInPicture",
                                @"value":@{@"enable": @(enable)}};
        [PLVWebViewManager sharedManager].autoSmallWindowPlay = enable;
        [[PLVWebViewManager sharedManager] callHandlerCallWebViewEvent:parames];
    } else if ([eventName isEqualToString:@"setEnableBackgroundPictureInPicture"]) {
        // 设置是否开启后台画中画
        NSDictionary *dict = event[@"value"];
        BOOL enable = [dict[@"enable"] boolValue];
        [self saveEnableBackgroundPictureInPicture:enable];
        [PLVWebViewManager sharedManager].autoSmallWindowPlay = enable;
        [[PLVWebViewManager sharedManager] callHandlerCallWebViewEvent:event];
    } else {
        NSDictionary *dict = event[@"value"];
        NSString *callback = dict[@"callback"];
        [self.mainWebView callHandler:callback data:dict responseCallback:^(id  _Nonnull responseData) {
        }];
    }
}

#pragma mark UIContainerViewControllerCallbacks

- (void)didMoveToParentViewController:(UIViewController*)parent {
    [super didMoveToParentViewController:parent];
    if(!parent) {
        NSLog(@"离开了页面");
        if ([PLVWebViewManager sharedManager].autoSmallWindowPlay) {
            [[PLVWebViewManager sharedManager] changeToFloatingWindow];
        }
    }
}

@end
