//
//  PLVViewController.m
//  WebView-Test
//
//  Created by ftao on 27/04/2018.
//  Copyright © 2018 PLV. All rights reserved.
//

#import "PLVURLInputViewController.h"
#import "PLVFloatingWebViewController.h"
#import <PLVWebViewSDK/PLVWebViewSDK.h>

static NSString *kTestUrlString = @"";

@interface PLVURLInputViewController ()

@property (nonatomic, copy) NSString * _Nullable customUserAgent;
@property (weak, nonatomic) IBOutlet UITextView *urlTextView;
@property (weak, nonatomic) IBOutlet UITextView *uaTextView;

@end

@implementation PLVURLInputViewController

#pragma mark - Lift cycle

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [[PLVConfigManager sharedManager] addHostWhitelist:@[@"demo.polyv.net"]];

    self.urlTextView.text = kTestUrlString;
    self.uaTextView.text = [PLVWebViewManager sharedManager].defaultUserAgent;
}

#pragma mark - [ Override ]

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    return UIInterfaceOrientationMaskPortrait;
}

- (BOOL)shouldAutorotate {
    return YES;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    self.navigationController.navigationBar.translucent = YES;
    [PLVWebViewManager sharedManager].supportedInterfaceOrientations = UIInterfaceOrientationMaskPortrait;
    [PLVWVFdUtil changeDeviceOrientation:UIDeviceOrientationPortrait];
}

#pragma mark - [ Event ]

#pragma mark - Action

-(IBAction)clickOpenWebViewAction:(id)sender {
    [self.view endEditing:YES];
    
    __weak typeof(self) weakSelf = self;
    // 通过block传递webView配置
    [PLVWebViewManager sharedManager].webViewConfigBlock = ^(PLVWKWebView * _Nonnull mainWebView, PLVWKWebView * _Nonnull subWebView) {
        if (weakSelf.customUserAgent && weakSelf.customUserAgent.length > 0) {
            mainWebView.customUserAgent = weakSelf.customUserAgent;
            subWebView.customUserAgent = weakSelf.customUserAgent;
        }
    };
    
    NSString *urlString = self.urlTextView.text;
    if (urlString && [urlString isKindOfClass:NSString.class] && urlString.length > 0) {
        urlString = [urlString stringByReplacingOccurrencesOfString:@"\n" withString:@""];
        PLVFloatingWebViewController *floatingWebVC = [[PLVFloatingWebViewController alloc] initWithURLString:urlString];
        [self.navigationController pushViewController:floatingWebVC animated:YES];
    }
}

- (IBAction)clickResetUAAction:(UIButton *)sender {
    self.customUserAgent = nil;
    self.uaTextView.text = [PLVWebViewManager sharedManager].defaultUserAgent;
}

- (IBAction)clickChangeUAAction:(UIButton *)sender {
    NSString *uaText = self.uaTextView.text;
    if (uaText && [uaText isKindOfClass:NSString.class] && uaText.length > 0) {
        self.customUserAgent = [uaText stringByReplacingOccurrencesOfString:@"\n" withString:@""];
    }
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [self.view endEditing:YES];
}

@end
