# 快速集成

## 1 阅读准备

提前下载项目Demo，准备好一个url，最好是保利威云直播/云点播的观看端url，可以体验到更完整的功能。



## 2 环境要求

| 名称·     | 要求   |
| --------- | ------ |
| iOS系统   | 9.0+   |
| CocoaPods | 1.7.0+ |
| Xcode     | 11.0+  |



## 3 项目配置

### 3.1 配置支持系统版本

打开项目的 PROJECT - Deployment Target - iOS Deployment Target 改为 9.0 或更高。

打开项目的 TARGETS - General - Deployment Info，把 iOS 系统改为 9.0 或更高。



### 3.2 配置 App Transport Security (ATS)

打开项目的 `info.plist `文件，添加如下内容：

```xml
<key>NSAppTransportSecurity</key>
<dict>
	<key>NSAllowsArbitraryLoads</key>
	<true/>
	<key>NSAllowsArbitraryLoadsInWebContent</key>
	<true/>
</dict>
```



### 3.3 配置 Bitcode

打开项目的 TARGETS - Build Settings，搜索 Enable Bitcode，并设置为 NO。



### 3.4 配置设备旋转

播放器支持全屏播放，需要在 TARGETS - General - Deployment Info 中，勾选支持的横屏旋转方向。



### 3.5 配置后台播放和画中画

打开项目的 TARGETS - Signings & Capabilities，点击 + Capability，选择 Background Modes，勾选 ‘Audio, AirPlay, and Picture in Picture’。



## 4 集成SDK

在 Podfile 文件中，添加以下内容：

```ruby
pod 'PLVWebViewSDK', '~> 1.0.0'
```

并在终端执行 `pod install`。

注意：我们强烈建议您带上版本号，避免自动升级。



## 5 WebView接入

### 5.1 界面初始化

在想要展示 WebView 的控制器页面，导入 SDK 头文件，引入 `PLVWebViewControllerProtocol` 协议，并定义初始化方法 `'+webViewControllerWithURLString:'`，示例代码如下：

```objective-c
// PLVFloatingWebViewController.h

#import <PLVWebViewSDK/PLVWebViewSDK.h>

@interface PLVFloatingWebViewController : UIViewController<
PLVWebViewControllerProtocol
>
    
@property (nonatomic, copy) NSString *urlString;

+ (instancetype)webViewControllerWithURLString:(NSString *)urlString;

@end
```

实现初始化方法 '+webViewControllerWithURLString:'，并在初始化方法中调用 `PLVWebViewManager` 的 `'-setupWebViewController:'` 方法， 示例代码如下：

```objective-c
// PLVFloatingWebViewController.m
    
@implementation PLVFloatingWebViewController

+ (instancetype)webViewControllerWithURLString:(NSString *)urlString {
    return [[PLVFloatingWebViewController alloc] initWithURLString:urlString];
}

- (instancetype)initWithURLString:(NSString *)urlString {
    self = [super init];
    if (self) {
        if(urlString && [urlString isKindOfClass:[NSString class]] && urlString.length > 0) {
            self.urlString = urlString;
        }
        
        [[PLVWebViewManager sharedManager] setupWebViewController:self];
    }
    return self;
}

@end
```

**注意，如果不这么配置，会导致页面销毁后无法从浮窗还原控制器页面。**



### 5.2 WebView 嵌入

在页面中定义用来布局 `webView` 控件的容器视图 `webviewParentView`，并调用 `PLVWebViewManager` 的 `'-attachWebViewInParentView:webViewUrl:'` 方法嵌入 `webView` 控件并在webView中打开urlString，示例代码如下：

```objective-c
// PLVFloatingWebViewController.m

@interface PLVFloatingWebViewController ()

@property (nonatomic, strong) UIView *webviewParentView; // 负责展示webview的容器视图

@end
    
@implementation PLVFloatingWebViewController
    
- (void)viewDidLoad {
    [super viewDidLoad];
    
    // 初始化容器视图
    self.webviewParentView = [[UIView alloc] init];
    [self.view addSubview:self.webviewParentView];
    
    // 嵌入WebView到容器视图中，并在webView中打开urlString
    [[PLVWebViewManager sharedManager] attachWebViewInParentView:self.webviewParentView webViewUrl:self.urlString];
}

// 容器视图布局
- (void)viewWillLayoutSubviews {
    self.webviewParentView.frame = self.view.bounds;
}

@end
```



### 5.3 界面销毁

在展示 WebView 的控制器页面即将销毁时，调用 `PLVWebViewManager` 的 `'-webViewDealloc'` 销毁 `webView` 控件，示例代码如下：

```objective-c
// PLVFloatingWebViewController.m

@implementation PLVFloatingWebViewController

- (void)dealloc {
    [[PLVWebViewManager sharedManager] webViewDealloc];
}
    
@end
```

**注意，如果不销毁会导致 `webView` 控件一直存在于应用后台并导致一系列的问题。**