# 高级功能

### 1 修改webView配置

在调用 `PLVWebViewManager` 的 `'-attachWebViewInParentView:webViewUrl:'` 方法之前，使用 `webViewConfigBlock` 进行 WebView 控件的个性化配置，其中 `PLVWKWebView` 是 `WKWebView` 的子类。

```objective-c
// 通过block传递webView配置
[PLVWebViewManager sharedManager].webViewConfigBlock = ^(PLVWKWebView * _Nonnull mainWebView, PLVWKWebView * _Nonnull subWebView) {
    // 在block里对PLVWKWebView实例对象进行个性化配置
};
```

其中，属性 `defaultUserAgent` 是指WebView 控件默认的 UserAgent，客户可根据需要，在 `defaultUserAgent` 的基础上进行修改，并通过 `webViewConfigBlock` 配置：

```objective-c
NSString *customUserAgent = [PLVWebViewManager sharedManager].defaultUserAgent;
// 根据需要修改 customUserAgent 的值
// ...
[PLVWebViewManager sharedManager].webViewConfigBlock = ^(PLVWKWebView * _Nonnull mainWebView, PLVWKWebView * _Nonnull subWebView) {
    mainWebView.customUserAgent = customUserAgent;
    subWebView.customUserAgent = customUserAgent;
};
```

### 2 Native - JS 通信

保利威 `webView` 控件提供了 Native 和 JS 之间通信的能力。如果是加载其他的 url 并且需要做通信时，就可以使用到这些通信的方法，示例代码如下：

```objective-c
[PLVWebViewManager sharedManager].webViewConfigBlock = ^(PLVWKWebView * _Nonnull mainWebView, PLVWKWebView * _Nonnull subWebView) {
		[mainWebView registerHandler:@"testObjcCallback" handler:^(id  _Nonnull data, PLVWVJBResponseCallback  _Nonnull responseCallback) {
            
     }];
     [mainWebView callHandler:@"testJavascriptHandler" data:nil responseCallback:^(id  _Nonnull responseData) {
            
     }];
};
```



### 3 浮窗功能开关

浮窗开关 `allowFloatingWindow` 默认是打开的，如果不需要悬浮窗，也可以通过该属性进行关闭。

```objective-c
[PLVWebViewManager sharedManager].allowFloatingWindow = NO;
```

但不是所有网址都支持悬浮窗，而是只有保利威的直播/点播观看url才能支持，想知道是否当前是否支持浮窗，可通过属性 `supportFloatingWindow` 获取，注意，如果在调用 `'-attachWebViewInParentView:webViewUrl:'` 方法之前，属性 `supportFloatingWindow` 为 `NO`。

```objective-c
BOOL supportFloatingWindow = [PLVWebViewManager sharedManager].supportFloatingWindow;
```



### 4 浮窗操作

除了通过保利威的网页唤起浮窗，也可以通过调用 `-changeToFloatingWindow` 方法主动唤起浮窗。

```objective-c
[[PLVWebViewManager sharedManager] changeToFloatingWindow];
```

同样可以调用 `-postRevertWindowToNormal` 、 `-closeWindow` 方法从浮窗恢复、关闭浮窗。

```objective-c
// 从浮窗恢复
[[PLVWebViewManager sharedManager] postRevertWindowToNormal];
// 关闭浮窗
[[PLVWebViewManager sharedManager] closeWindow];
```



### 5 浮窗配置

我们还可以对浮窗的大小、默认位置、以及样式进行配置修改：

```objective-c
// 配置浮窗大小
[[PLVWebViewManager sharedManager] setFloatingWindowSize:size];
// 配置浮窗初始位置
[[PLVWebViewManager sharedManager] setFloatingWindowLocation:point];
// 应用内小窗样式修改
UIView *appWindowView = [PLVWebViewManager sharedManager].appWindowView;
appWindowView.layer.cornerRadius = 8.0f;
appWindowView.layer.masksToBounds = YES;
```

其中，调用 `-changeToFloatingWindow` 方法主动唤起的浮窗，默认宽度为屏幕的0.3倍，宽高比16:9。通过保利威的网页唤起浮窗，浮窗大小由网页告知。初始位置是指浮窗右下角距离屏幕右下角的相对位置，默认为(-10, -100)， 表示悬浮窗右边沿距离屏幕右边沿10pt，下边沿距离屏幕下边沿100pt。



### 6 浮窗相关回调监听

如果希望知道网页何时唤起浮窗，用户何时点击浮窗按钮进行还原或关闭，可在想要展示 WebView 的控制器页面配置监听：

```objective-c
@interface PLVFloatingWebViewController ()<
PLVWebViewManagerDelegate
>
    
@end

@implementation PLVFloatingWebViewController

- (instancetype)initWithURLString:(NSString *)urlString {
    self = [super init];
    if (self) {
        if(urlString && [urlString isKindOfClass:[NSString class]] && urlString.length > 0) {
            self.urlString = urlString;
        }
        
        [[PLVWebViewManager sharedManager] setupWebViewController:self];
        // 配置监听
        [PLVWebViewManager sharedManager].delegate = self;
    }
    return self;
}

// h5页面即将唤起浮窗
- (void)webviewManager:(PLVWebViewManager *)manager willJumpLink:(NSString *)link data:(NSDictionary *)data {
    
}

// 用户点击浮窗的还原按钮，即将还原浮窗
- (void)webviewManagerDidRecoverWindow:(PLVWebViewManager *)manager {
    
}

// 用户点击浮窗的关闭按钮，即将关闭浮窗
- (void)webviewManagerDidCloseWindow:(PLVWebViewManager *)manager {
    
}

@end
```

### 7 系统小窗画中画的开启

系统小窗在iOS中就是画中画功能，需要设置 setFloatingWindowType 即可在开启小窗时 显示为系统小窗。

```objective-c
[[PLVWebViewManager sharedManager] setFloatingWindowType:YES];
```

系统小窗开启需要的条件有

1、在iOS14+的系统才支持画中画

2、需要直播中的房间才可以开启，回放视频不支持画中画



最后附上 PLVWebViewManager.h 源码，里面对于各个接口、属性都有详细的注释：

```objective-c
#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

@class PLVWKWebView;

@protocol PLVWebViewControllerProtocol, PLVWebViewManagerDelegate;

typedef void (^PLVWebViewConfigBlock)(PLVWKWebView *mainWebView, PLVWKWebView *subWebView);

/// PLVWebViewSDK 核心类，支持以下功能：
/// 1. 支持适配保利威的直播页面
/// 2. 支持浮窗展示保利威的直播页面
/// 3. 也支持作为其他h5网页的容器
@interface PLVWebViewManager : NSObject

#pragma mark 可配置属性
/// 委托监听
@property (nonatomic, weak) id<PLVWebViewManagerDelegate> delegate;
/// 是否要使用浮窗功能，默认为YES
@property (nonatomic, assign) BOOL allowFloatingWindow;
/// webView配置Block
@property (nonatomic, copy) PLVWebViewConfigBlock webViewConfigBlock;

#pragma mark 只读属性

/// webView默认user-agent
@property (nonatomic, copy, readonly) NSString *defaultUserAgent;
/// 当前打开网页是否支持浮窗功能
@property (nonatomic, assign, readonly) BOOL supportFloatingWindow;

/// webView 管理器单例类
+ (instancetype)sharedManager;

/// 添加webView到parentView中，并打开url为webViewUrl的网页
- (void)attachWebViewInParentView:(UIView *)parentView webViewUrl:(NSString *)webViewUrl;

/// mainWebview所在控制器页面（还原小窗时需要用到）
/// @note 进入页面时调用
- (void)setupWebViewController:(id<PLVWebViewControllerProtocol> _Nullable)webViewController;

/// 页面销毁时调用该方法回收不需要的webView
- (void)webViewDealloc;

/// 主动切换到悬浮窗
/// @note 当allowFloatingWindow为YES、supportFloatingWindow也为YES时才会切换
- (BOOL)changeToFloatingWindow;

/// 主动恢复为非悬浮窗
/// @note 主动调用不触发相关回调
- (void)postRevertWindowToNormal;

/// 主动关闭悬浮窗
/// @note 主动调用不触发相关回调
- (void)closeWindow;

/// 设置悬浮窗大小，宽高均应为正数，如不设置或参数不规范，则使用默认值
/// @param size 悬浮窗的宽高，
/// @note 如不设置或参数不规范，主动唤出浮窗时浮窗宽度为屏幕的0.3倍，宽高比16:9，js唤出浮窗时以h5告知的宽高为准
- (void)setFloatingWindowSize:(CGSize)size;

/// 设置悬浮窗位置，如不设置则使用默认值
/// @param point 悬浮窗右下角距离屏幕右下角的相对位置，默认为(-10, -100), 表示悬浮窗右边沿距离屏幕右边沿10pt，下边沿距离屏幕下边沿100pt
- (void)setFloatingWindowLocation:(CGPoint)point;

@end

@protocol PLVWebViewControllerProtocol <NSObject>

/// 展示h5的控制器需要实现的方法，可用于页面被销毁时重建控制器
+ (instancetype)webViewControllerWithURLString:(NSString *)urlString;

@end

@protocol PLVWebViewManagerDelegate <NSObject>

@optional

/// 点击商品，打开商品页时触发
/// @param link subWebview新页面的链接
/// @param data 第三方应用信息(客户自定义使用)
- (void)webviewManager:(PLVWebViewManager *)manager willJumpLink:(NSString *)link data:(NSDictionary *)data;

/// 点击小窗区域，从小窗中恢复到页面时触发该回调
- (void)webviewManagerDidRecoverWindow:(PLVWebViewManager *)manager;

/// 点击小窗区域，关闭小窗时触发该回调
- (void)webviewManagerDidCloseWindow:(PLVWebViewManager *)manager;

@end
```

